@@ -6,6 +6,7 @@ use musli::de::{
     Decoder, PackDecoder, PairDecoder, PairsDecoder, SequenceDecoder, ValueVisitor, VariantDecoder,
 };
 use musli::error::Error;
+use musli::never::Never;
 use musli_common::reader::PosReader;
 
 /// A very simple decoder suitable for storage decoding.
@@ -54,13 +55,13 @@ where
     L: UsizeEncoding,
 {
     type Error = R::Error;
+    type Buffer = Never<R::Error>;
     type Pack = Self;
     type Some = Self;
     type Sequence = LimitedStorageDecoder<R, I, L>;
     type Tuple = Self;
     type Map = LimitedStorageDecoder<R, I, L>;
     type Struct = LimitedStorageDecoder<R, I, L>;
-    type TupleStruct = LimitedStorageDecoder<R, I, L>;
     type Variant = Self;
 
     #[inline]
@@ -124,6 +125,7 @@ where
                 self.0.expecting(f)
             }
 
+            #[cfg(feature = "std")]
             #[inline]
             fn visit_owned(self, bytes: Vec<u8>) -> Result<Self::Ok, Self::Error> {
                 let string = String::from_utf8(bytes).map_err(Self::Error::custom)?;
@@ -269,16 +271,6 @@ where
         LimitedStorageDecoder::new(self)
     }
 
-    #[inline]
-    fn decode_tuple_struct(self, _: usize) -> Result<Self::TupleStruct, Self::Error> {
-        LimitedStorageDecoder::new(self)
-    }
-
-    #[inline]
-    fn decode_unit_struct(self) -> Result<(), Self::Error> {
-        self.decode_unit()
-    }
-
     #[inline]
     fn decode_variant(self) -> Result<Self::Variant, Self::Error> {
         Ok(self)
@@ -298,6 +290,11 @@ where
     fn next(&mut self) -> Result<Self::Decoder<'_>, Self::Error> {
         Ok(StorageDecoder::new(self.reader.pos_borrow_mut()))
     }
+
+    #[inline]
+    fn end(self) -> Result<(), Self::Error> {
+        Ok(())
+    }
 }
 
 impl<'de, R, I, L> LimitedStorageDecoder<R, I, L>
@@ -338,6 +335,11 @@ where
             self.decoder.reader.pos_borrow_mut(),
         )))
     }
+
+    #[inline]
+    fn end(self) -> Result<(), Self::Error> {
+        Ok(())
+    }
 }
 
 impl<'de, R, I, L> PairsDecoder<'de> for LimitedStorageDecoder<R, I, L>
@@ -368,6 +370,11 @@ where
             self.decoder.reader.pos_borrow_mut(),
         )))
     }
+
+    #[inline]
+    fn end(self) -> Result<(), Self::Error> {
+        Ok(())
+    }
 }
 
 impl<'de, R, I, L> PairDecoder<'de> for StorageDecoder<R, I, L>
