@@ -45,7 +45,6 @@ where
     type Tuple = Self;
     type Map = Self;
     type Struct = Self;
-    type TupleStruct = Self;
     type Variant = Self;
 
     #[inline]
@@ -194,7 +193,7 @@ where
 
     #[inline]
     fn encode_tuple(self, _: usize) -> Result<Self::Sequence, Self::Error> {
-        // NB: tuple has statically known fixed length.
+        // NB: A tuple has statically known fixed length.
         Ok(self)
     }
 
@@ -205,23 +204,11 @@ where
     }
 
     #[inline]
-    fn encode_struct(mut self, fields: usize) -> Result<Self::Struct, Self::Error> {
-        L::encode_usize(self.writer.borrow_mut(), fields)?;
-        Ok(self)
-    }
-
-    #[inline]
-    fn encode_tuple_struct(mut self, len: usize) -> Result<Self::TupleStruct, Self::Error> {
+    fn encode_struct(mut self, len: usize) -> Result<Self::Struct, Self::Error> {
         L::encode_usize(self.writer.borrow_mut(), len)?;
         Ok(self)
     }
 
-    #[inline]
-    fn encode_unit_struct(mut self) -> Result<Self::Ok, Self::Error> {
-        L::encode_usize(self.writer.borrow_mut(), 0)?;
-        Ok(())
-    }
-
     #[inline]
     fn encode_variant(self) -> Result<Self::Variant, Self::Error> {
         Ok(self)
@@ -279,7 +266,7 @@ where
     type Ok = ();
     type Error = W::Error;
     type First<'this> = StorageEncoder<W::Mut<'this>, I, L> where Self: 'this;
-    type Second = Self;
+    type Second<'this> = StorageEncoder<W::Mut<'this>, I, L> where Self: 'this;
 
     #[inline]
     fn first(&mut self) -> Result<Self::First<'_>, Self::Error> {
@@ -287,8 +274,13 @@ where
     }
 
     #[inline]
-    fn second(self) -> Result<Self::Second, Self::Error> {
-        Ok(self)
+    fn second(&mut self) -> Result<Self::Second<'_>, Self::Error> {
+        Ok(StorageEncoder::new(self.writer.borrow_mut()))
+    }
+
+    #[inline]
+    fn end(self) -> Result<Self::Ok, Self::Error> {
+        Ok(())
     }
 }
 
